@@ -1006,7 +1006,8 @@ static void usage(void)
 int main(int argc, char **argv)
 {
 	static struct option long_options[] = {
-		{"help", 0, 0, 'h'}
+		{"help", 0, 0, 'h'},
+		{ 0 , 0 , 0, 0 }
 	};
 	int c;
 	snd_config_t *conf;
