@@ -73,6 +73,14 @@
 #define _snd_seq_remove_events sndrv_seq_remove_events
 #define _snd_instr_header sndrv_seq_instr_header
 
+#define sndrv_seq_addr	snd_seq_addr
+#define sndrv_seq_tick_time_t	snd_seq_tick_time_t
+#define sndrv_seq_real_time	snd_seq_real_time
+#define sndrv_seq_timestamp	snd_seq_timestamp
+#define sndrv_seq_instr		snd_seq_instr
+#define sndrv_seq_event		snd_seq_event
+#define sndrv_seq_instr_cluster_t	snd_seq_instr_cluster_t
+
 #if 0
 typedef struct sndrv_seq_addr snd_seq_addr_t;
 #define snd_seq_tick_time_t sndrv_seq_tick_time_t
@@ -96,7 +104,6 @@ typedef struct sndrv_seq_event snd_seq_event_t;
 
 #include <sound/asound.h>
 #include <sound/asoundef.h>
-#include <sound/asequencer.h>
 #include "alsa-symbols.h"
 #include "version.h"
 #include "global.h"
@@ -113,6 +120,7 @@ typedef struct sndrv_seq_event snd_seq_event_t;
 #include "mixer.h"
 #include "seq_event.h"
 #include "seq.h"
+#include <sound/asequencer.h>
 #include "seqmid.h"
 #include "seq_midi_event.h"
 #include "conv.h"
@@ -228,7 +236,11 @@ static inline int snd_open_device(const char *filename, int fmode)
 	int fd = open(filename, fmode);
 	if (fd >= 0)
 		return fd;
-	return rsm_open_device(filename, fmode);
+	if (errno == EAGAIN || errno == EBUSY)
+		return fd;
+	if (! access(filename, F_OK))
+		return rsm_open_device(filename, fmode);
+	return -1;
 }
 #else
 #define snd_open_device(filename, fmode) open(filename, fmode);
@@ -239,4 +251,9 @@ void *snd_dlobj_cache_lookup(const char *name);
 int snd_dlobj_cache_add(const char *name, void *dlobj, void *open_func);
 void snd_dlobj_cache_cleanup(void);
 
+/* for recursive checks */
+void snd_config_set_hop(snd_config_t *conf, int hop);
+int snd_config_check_hop(snd_config_t *conf);
+#define SND_CONF_MAX_HOPS	64
+
 #endif
