@@ -1,5 +1,5 @@
 /**
- * \file <alsa/seqmid.h>
+ * \file include/seqmid.h
  * \brief Application interface library for the ALSA driver
  * \author Jaroslav Kysela <perex@suse.cz>
  * \author Abramo Bagnara <abramo@alsa-project.org>
@@ -7,8 +7,8 @@
  * \date 1998-2001
  *
  * Application interface library for the ALSA driver
- *
- *
+ */
+/*
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Lesser General Public License as
  *   published by the Free Software Foundation; either version 2.1 of
@@ -42,15 +42,31 @@ extern "C" {
 /**
  * \brief initialize event record
  * \param ev event record pointer
+ * 
+ * This macro clears the given event record pointer to the default status.
  */
 #define snd_seq_ev_clear(ev) \
 	memset(ev, 0, sizeof(snd_seq_event_t))
 
+/**
+ * \brief set the tag for given event
+ * \param ev event record
+ * \param t event tag
+ *
+ * This macro sets the tag to the given event record.
+ */
+#define snd_seq_ev_set_tag(ev,t) \
+	((ev)->tag = (t))
+
 /**
  * \brief set the explicit destination
  * \param ev event record
  * \param c destination client id
  * \param p destination port id
+ *
+ * This macro sets the client and port id numbers to the given event record.
+ *
+ * \sa snd_seq_ev_set_subs()
  */
 #define snd_seq_ev_set_dest(ev,c,p) \
 	((ev)->dest.client = (c), (ev)->dest.port = (p))
@@ -58,6 +74,10 @@ extern "C" {
 /**
  * \brief set broadcasting to subscribers
  * \param ev event record
+ *
+ * This macro sets the destination as the subscribers.
+ *
+ * \sa snd_seq_ev_set_dest()
  */
 #define snd_seq_ev_set_subs(ev) \
 	((ev)->dest.client = SND_SEQ_ADDRESS_SUBSCRIBERS,\
@@ -66,6 +86,10 @@ extern "C" {
 /**
  * \brief set broadcasting to all clients/ports
  * \param ev event record
+ *
+ * This macro sets the destination as the broadcasting.
+ *
+ * \sa snd_seq_ev_set_dest()
  */
 #define snd_seq_ev_set_broadcast(ev) \
 	((ev)->dest.client = SND_SEQ_ADDRESS_BROADCAST,\
@@ -75,6 +99,8 @@ extern "C" {
  * \brief set the source port
  * \param ev event record
  * \param p source port id
+ *
+ * This macro sets the source port id number.
  */
 #define snd_seq_ev_set_source(ev,p) \
 	((ev)->source.port = (p))
@@ -82,6 +108,11 @@ extern "C" {
 /**
  * \brief set direct passing mode (without queued)
  * \param ev event instance
+ *
+ * This macro sets the event to the direct passing mode
+ * to be delivered immediately without queueing.
+ * 
+ * \sa snd_seq_ev_schedule_tick(), snd_seq_ev_schedule_real()
  */
 #define snd_seq_ev_set_direct(ev) \
 	((ev)->queue = SND_SEQ_QUEUE_DIRECT)
@@ -92,6 +123,11 @@ extern "C" {
  * \param q queue id to schedule
  * \param relative relative time-stamp if non-zero
  * \param ttick tick time-stamp to be delivered
+ *
+ * This macro sets the scheduling of the event in the
+ * MIDI tick mode.
+ *
+ * \sa snd_seq_ev_schedule_real(), snd_seq_ev_set_direct()
  */
 #define snd_seq_ev_schedule_tick(ev, q, relative, ttick) \
 	((ev)->flags &= ~(SND_SEQ_TIME_STAMP_MASK | SND_SEQ_TIME_MODE_MASK),\
@@ -106,6 +142,11 @@ extern "C" {
  * \param q queue id to schedule
  * \param relative relative time-stamp if non-zero
  * \param rtime time-stamp to be delivered
+ *
+ * This macro sets the scheduling of the event in the
+ * realtime mode.
+ *
+ * \sa snd_seq_ev_schedule_tick(), snd_seq_ev_set_direct()
  */
 #define snd_seq_ev_schedule_real(ev, q, relative, rtime) \
 	((ev)->flags &= ~(SND_SEQ_TIME_STAMP_MASK | SND_SEQ_TIME_MODE_MASK),\
@@ -128,6 +169,8 @@ extern "C" {
  * \param ev event instance
  *
  * Sets the event length mode as fixed size.
+ *
+ * \sa snd_seq_ev_set_variable(), snd_seq_ev_set_varusr()
  */
 #define snd_seq_ev_set_fixed(ev) \
 	((ev)->flags &= ~SND_SEQ_EVENT_LENGTH_MASK,\
@@ -140,6 +183,8 @@ extern "C" {
  * \param dataptr pointer of the external data
  *
  * Sets the event length mode as variable length and stores the data.
+ *
+ * \sa snd_seq_ev_set_fixed(), snd_seq_ev_set_varusr()
  */
 #define snd_seq_ev_set_variable(ev, datalen, dataptr) \
 	((ev)->flags &= ~SND_SEQ_EVENT_LENGTH_MASK,\
@@ -150,10 +195,12 @@ extern "C" {
 /**
  * \brief set varusr data
  * \param ev event instance
- * \param len length of the external data
- * \param ptr pointer of the external data
+ * \param datalen length of the external data
+ * \param dataptr pointer of the external data
  *
  * Sets the event length mode as variable user-space data and stores the data.
+ *
+ * \sa snd_seq_ev_set_fixed(), snd_seq_ev_set_variable()
  */
 #define snd_seq_ev_set_varusr(ev, datalen, dataptr) \
 	((ev)->flags &= ~SND_SEQ_EVENT_LENGTH_MASK,\
@@ -178,6 +225,8 @@ extern "C" {
  * \brief set the start queue event
  * \param ev event record
  * \param q queue id to start
+ *
+ * \sa snd_seq_ev_set_queue_stop(), snd_seq_ev_set_queue_continue()
  */
 #define snd_seq_ev_set_queue_start(ev, q) \
 	snd_seq_ev_set_queue_control(ev, SND_SEQ_EVENT_START, q, 0)
@@ -186,6 +235,8 @@ extern "C" {
  * \brief set the stop queue event
  * \param ev event record
  * \param q queue id to stop
+ *
+ * \sa snd_seq_ev_set_queue_start(), snd_seq_ev_set_queue_continue()
  */
 #define snd_seq_ev_set_queue_stop(ev, q) \
 	snd_seq_ev_set_queue_control(ev, SND_SEQ_EVENT_STOP, q, 0)
@@ -194,6 +245,8 @@ extern "C" {
  * \brief set the stop queue event
  * \param ev event record
  * \param q queue id to continue
+ *
+ * \sa snd_seq_ev_set_queue_start(), snd_seq_ev_set_queue_stop()
  */
 #define snd_seq_ev_set_queue_continue(ev, q) \
 	snd_seq_ev_set_queue_control(ev, SND_SEQ_EVENT_CONTINUE, q, 0)
