@@ -364,7 +364,7 @@ Several built-in functions are available.</P>
 
 <P>A function is defined with the id \c \@func and the function name. All other
 values in the current compound are used as configuration for the function.
-If the compound func.<function_name> is defined in the root node, then the
+If the compound func.\<function_name\> is defined in the root node, then the
 library and function from this compound configuration are used, otherwise
 'snd_func_' is prefixed to the string and code from the ALSA library is used.
 The definition of a function looks like:</P> 
@@ -415,7 +415,6 @@ beginning:</P>
 
 
 #include <stdarg.h>
-#include <wordexp.h>
 #include <dlfcn.h>
 #include <limits.h>
 #include <sys/stat.h>
@@ -441,6 +440,7 @@ struct _snd_config {
 	} u;
 	struct list_head list;
 	snd_config_t *father;
+	int hop;
 };
 
 struct filedesc {
@@ -499,16 +499,21 @@ static int safe_strtod(const char *str, double *val)
 	char *end;
 	double v;
 	char *saved_locale;
+	char locstr[64]; /* enough? */
 	int err;
 
 	if (!*str)
 		return -EINVAL;
 	saved_locale = setlocale(LC_NUMERIC, NULL);
-	setlocale(LC_NUMERIC, "C");
+	if (saved_locale) {
+		snprintf(locstr, sizeof(locstr), "%s", saved_locale);
+		setlocale(LC_NUMERIC, "C");
+	}
 	errno = 0;
 	v = strtod(str, &end);
 	err = -errno;
-	setlocale(LC_NUMERIC, saved_locale);
+	if (saved_locale)
+		setlocale(LC_NUMERIC, locstr);
 	if (err)
 		return err;
 	if (*end)
@@ -671,29 +676,75 @@ static int get_quotedchar(input_t *input)
 	}
 }
 
+#define LOCAL_STR_BUFSIZE	64
+struct local_string {
+	char *buf;
+	size_t alloc;
+	size_t idx;
+	char tmpbuf[LOCAL_STR_BUFSIZE];
+};
+
+static void init_local_string(struct local_string *s)
+{
+	memset(s, 0, sizeof(*s));
+	s->buf = s->tmpbuf;
+	s->alloc = LOCAL_STR_BUFSIZE;
+}
+
+static void free_local_string(struct local_string *s)
+{
+	if (s->buf != s->tmpbuf)
+		free(s->buf);
+}
+
+static int add_char_local_string(struct local_string *s, int c)
+{
+	if (s->idx >= s->alloc) {
+		size_t nalloc = s->alloc * 2;
+		if (s->buf == s->tmpbuf) {
+			s->buf = malloc(nalloc);
+			if (s->buf == NULL)
+				return -ENOMEM;
+			memcpy(s->buf, s->tmpbuf, s->alloc);
+		} else {
+			char *ptr = realloc(s->buf, nalloc);
+			if (ptr == NULL)
+				return -ENOMEM;
+			s->buf = ptr;
+		}
+		s->alloc = nalloc;
+	}
+	s->buf[s->idx++] = c;
+	return 0;
+}
+
+static char *copy_local_string(struct local_string *s)
+{
+	char *dst = malloc(s->idx + 1);
+	if (dst) {
+		memcpy(dst, s->buf, s->idx);
+		dst[s->idx] = '\0';
+	}
+	return dst;
+}
+
 static int get_freestring(char **string, int id, input_t *input)
 {
-	const size_t bufsize = 64;
-	char _buf[bufsize];
-	char *buf = _buf;
-	size_t alloc = bufsize;
-	size_t idx = 0;
+	struct local_string str;
 	int c;
+
+	init_local_string(&str);
 	while (1) {
 		c = get_char(input);
 		if (c < 0) {
 			if (c == LOCAL_UNEXPECTED_EOF) {
-				char *s = malloc(idx + 1);
-				if (!s)
-					return -ENOMEM;
-				memcpy(s, buf, idx);
-				s[idx] = '\0';
-				*string = s;
-				c = 0;
+				*string = copy_local_string(&str);
+				if (! *string)
+					c = -ENOMEM;
+				else
+					c = 0;
 			}
-			if (alloc > bufsize)
-				free(buf);
-			return c;
+			break;
 		}
 		switch (c) {
 		case '.':
@@ -715,95 +766,58 @@ static int get_freestring(char **string, int id, input_t *input)
 		case '"':
 		case '\\':
 		case '#':
-		{
-			char *s = malloc(idx + 1);
-			if (!s)
-				return -ENOMEM;
-			unget_char(c, input);
-			memcpy(s, buf, idx);
-			s[idx] = '\0';
-			*string = s;
-			if (alloc > bufsize)
-				free(buf);
-			return 0;
-		}
+			*string = copy_local_string(&str);
+			if (! *string)
+				c = -ENOMEM;
+			else {
+				unget_char(c, input);
+				c = 0;
+			}
+			goto _out;
 		default:
 			break;
 		}
-		if (idx >= alloc) {
-			size_t old_alloc = alloc;
-			alloc *= 2;
-			if (old_alloc == bufsize) {
-				buf = malloc(alloc);
-				if (buf == NULL)
-					return -ENOMEM;
-				memcpy(buf, _buf, old_alloc);
-			} else {
-				char *ptr = realloc(buf, alloc);
-				if (ptr == NULL) {
-					free(buf);
-					return -ENOMEM;
-				}
-				buf = ptr;
-			}
+		if (add_char_local_string(&str, c) < 0) {
+			c = -ENOMEM;
+			break;
 		}
-		buf[idx++] = c;
 	}
-	return 0;
+ _out:
+	free_local_string(&str);
+	return c;
 }
 			
 static int get_delimstring(char **string, int delim, input_t *input)
 {
-	const size_t bufsize = 64;
-	char _buf[bufsize];
-	char *buf = _buf;
-	size_t alloc = bufsize;
-	size_t idx = 0;
+	struct local_string str;
 	int c;
+
+	init_local_string(&str);
 	while (1) {
 		c = get_char(input);
 		if (c < 0)
-			return c;
-		switch (c) {
-		case '\\':
+			break;
+		if (c == '\\') {
 			c = get_quotedchar(input);
 			if (c < 0)
-				return c;
+				break;
 			if (c == '\n')
 				continue;
+		} else if (c == delim) {
+			*string = copy_local_string(&str);
+			if (! *string)
+				c = -ENOMEM;
+			else
+				c = 0;
 			break;
-		default:
-			if (c == delim) {
-				char *s = malloc(idx + 1);
-				if (!s)
-					return -ENOMEM;
-				memcpy(s, buf, idx);
-				s[idx] = '\0';
-				*string = s;
-				if (alloc > bufsize)
-					free(buf);
-				return 0;
-			}
 		}
-		if (idx >= alloc) {
-			size_t old_alloc = alloc;
-			alloc *= 2;
-			if (old_alloc == bufsize) {
-				buf = malloc(alloc);
-				if (buf == NULL)
-					return -ENOMEM;
-				memcpy(buf, _buf, old_alloc);
-			} else {
-				char *ptr = realloc(buf, alloc);
-				if (ptr == NULL) {
-					free(buf);
-					return -ENOMEM;
-				}
-				buf = ptr;
-			}
+		if (add_char_local_string(&str, c) < 0) {
+			c = -ENOMEM;
+			break;
 		}
-		buf[idx++] = c;
 	}
+	 free_local_string(&str);
+	 return c;
 }
 
 /* Return 0 for free string, 1 for delimited string */
@@ -966,8 +980,7 @@ static int parse_value(snd_config_t **_n, snd_config_t *father, input_t *input,
 		if (err < 0)
 			return err;
 	}
-	if (n->u.string)
-		free(n->u.string);
+	free(n->u.string);
 	n->u.string = s;
 	*_n = n;
 	return 0;
@@ -978,14 +991,14 @@ static int parse_array_defs(snd_config_t *farther, input_t *input, int skip, int
 
 static int parse_array_def(snd_config_t *father, input_t *input, int idx, int skip, int override)
 {
-	char static_id[12], *id = NULL;
+	char *id = NULL;
 	int c;
 	int err;
 	snd_config_t *n = NULL;
 
 	if (!skip) {
+		char static_id[12];
 		snprintf(static_id, sizeof(static_id), "%i", idx);
-		static_id[sizeof(static_id)-1] = '\0';
 		id = strdup(static_id);
 		if (id == NULL)
 			return -ENOMEM;
@@ -1042,8 +1055,7 @@ static int parse_array_def(snd_config_t *father, input_t *input, int idx, int sk
 	}
 	err = 0;
       __end:
-      	if (id)
-      		free(id);
+	free(id);
       	return err;
 }
 
@@ -1204,8 +1216,7 @@ static int parse_def(snd_config_t *father, input_t *input, int skip, int overrid
 		unget_char(c, input);
 	}
       __end:
-      	if (id)
-	      	free(id);
+	free(id);
 	return err;
 }
 		
@@ -1228,7 +1239,7 @@ static int parse_defs(snd_config_t *father, input_t *input, int skip, int overri
 
 static void string_print(char *str, int id, snd_output_t *out)
 {
-	unsigned char *p = str;
+	unsigned char *p = (unsigned char *)str;
 	if (!id) {
 		switch (*p) {
 		case 0:
@@ -1268,7 +1279,7 @@ static void string_print(char *str, int id, snd_output_t *out)
 	return;
  quoted:
 	snd_output_putc(out, '\'');
-	p = str;
+	p = (unsigned char *)str;
 	while (*p) {
 		int c;
 		c = *p;
@@ -1432,8 +1443,7 @@ int snd_config_substitute(snd_config_t *dst, snd_config_t *src)
 		if (err < 0)
 			return err;
 	}
-	if (dst->id)
-		free(dst->id);
+	free(dst->id);
 	dst->id = src->id;
 	dst->type = src->type;
 	dst->u = src->u;
@@ -1514,8 +1524,7 @@ int snd_config_set_id(snd_config_t *config, const char *id)
 	new_id = strdup(id);
 	if (!new_id)
 		return -ENOMEM;
-	if (config->id)
-		free(config->id);
+	free(config->id);
 	config->id = new_id;
 	return 0;
 }
@@ -1682,16 +1691,14 @@ int snd_config_delete(snd_config_t *config)
 		break;
 	}
 	case SND_CONFIG_TYPE_STRING:
-		if (config->u.string)
-			free(config->u.string);
+		free(config->u.string);
 		break;
 	default:
 		break;
 	}
 	if (config->father)
 		list_del(&config->list);
-	if (config->id)
-		free(config->id);
+	free(config->id);
 	free(config);
 	return 0;
 }
@@ -2009,8 +2016,7 @@ int snd_config_set_string(snd_config_t *config, const char *value)
 	} else {
 		new_string = NULL;
 	}
-	if (config->u.string)
-		free(config->u.string);
+	free(config->u.string);
 	config->u.string = new_string;
 	return 0;
 }
@@ -2077,8 +2083,7 @@ int snd_config_set_ascii(snd_config_t *config, const char *ascii)
 			char *ptr = strdup(ascii);
 			if (ptr == NULL)
 				return -ENOMEM;
-			if (config->u.string)
-				free(config->u.string);
+			free(config->u.string);
 			config->u.string = ptr;
 		}
 		break;
@@ -2688,8 +2693,7 @@ static int snd_config_hooks_call(snd_config_t *root, snd_config_t *config, snd_c
 		if (err >= 0 && nroot)
 			err = snd_config_substitute(root, nroot);
 	}
-	if (buf)
-		free(buf);
+	free(buf);
 	if (err < 0)
 		return err;
 	return 0;
@@ -2830,11 +2834,10 @@ int snd_config_hook_load(snd_config_t *root, snd_config_t *config, snd_config_t
 	*dst = NULL;
 	err = 0;
        _err:
-	for (idx = 0; idx < fi_count; idx++)
-       		if (fi[idx].name)
-       			free(fi[idx].name);
-       	if (fi)
-       		free(fi);
+	if (fi)
+		for (idx = 0; idx < fi_count; idx++)
+			free(fi[idx].name);
+	free(fi);
 	snd_config_delete(n);
 	return err;
 }
@@ -2891,8 +2894,7 @@ int snd_config_hook_load_for_all_cards(snd_config_t *root, snd_config_t *config,
 		      __err:
 			if (private_data)
 				snd_config_delete(private_data);
-			if (fdriver)
-				free(fdriver);
+			free(fdriver);
 			if (err < 0)
 				return err;
 		}
@@ -3092,11 +3094,9 @@ int snd_config_update_free(snd_config_update_t *update)
 	unsigned int k;
 
 	assert(update);
-	assert(update->count > 0 && update->finfo);
 	for (k = 0; k < update->count; k++)
 		free(update->finfo[k].name);
-	if (update->finfo)
-		free(update->finfo);
+	free(update->finfo);
 	free(update);
 	return 0;
 }
@@ -3487,8 +3487,7 @@ static int _snd_config_evaluate(snd_config_t *src,
 			}
 		}
 	       _errbuf:
-		if (buf)
-			free(buf);
+		free(buf);
 		if (err < 0)
 			return err;
 		return 0;
@@ -3794,7 +3793,7 @@ static int parse_args(snd_config_t *subs, const char *str, snd_config_t *defs)
 		snd_config_t *def, *sub, *typ;
 		const char *new = str;
 		const char *tmp;
-		char *val;
+		char *val = NULL;
 		err = parse_arg(&new, &varlen, &val);
 		if (err < 0)
 			goto _err;
@@ -3999,6 +3998,24 @@ int snd_config_search_definition(snd_config_t *config,
 	return snd_config_expand(conf, config, args, NULL, result);
 }
 
+#ifndef DOC_HIDDEN
+void snd_config_set_hop(snd_config_t *conf, int hop)
+{
+	conf->hop = hop;
+}
+
+int snd_config_check_hop(snd_config_t *conf)
+{
+	if (conf) {
+		if (conf->hop >= SND_CONF_MAX_HOPS) {
+			SYSERR("Too many definition levels (looped?)\n");
+			return -EINVAL;
+		}
+		return conf->hop;
+	}
+	return 0;
+}
+#endif
 
 #if 0
 /* Not strictly needed, but useful to check for memory leaks */
