@@ -45,7 +45,7 @@
 #define ALSA_NAMES_PATH2 "~/.asoundnm"
 #endif
 
-static int names_parse(snd_config_t *top, const char *interface, snd_devname_t **list)
+static int names_parse(snd_config_t *top, const char *iface, snd_devname_t **list)
 {
 	snd_config_iterator_t i, next;
 	snd_config_iterator_t j, jnext;
@@ -54,7 +54,7 @@ static int names_parse(snd_config_t *top, const char *interface, snd_devname_t *
 	snd_devname_t *dn, *last = NULL;
 	int err;
 
-	err = snd_config_search(top, interface, &top);
+	err = snd_config_search(top, iface, &top);
 	if (err < 0)
 		return err;
 	snd_config_for_each(i, next, top) {
@@ -75,6 +75,7 @@ static int names_parse(snd_config_t *top, const char *interface, snd_devname_t *
 					err = -ENOMEM;
 					goto _err;
 				}
+				continue;
 			}
 			if (strcmp(id, "comment") == 0) {
 				err = snd_config_get_string(m, (const char **)&comment);
@@ -85,6 +86,7 @@ static int names_parse(snd_config_t *top, const char *interface, snd_devname_t *
 					err = -ENOMEM;
 					goto _err;
 				}
+				continue;
 			}
 		}
 		if (name != NULL) {
@@ -103,37 +105,34 @@ static int names_parse(snd_config_t *top, const char *interface, snd_devname_t *
 			}
 			last = dn;
 		} else {
-			if (comment != NULL)
-				free(comment);
+			free(comment);
 		}
 	}
 	return 0;
 
        _err:
-      	if (name)
-      		free(name);
-      	if (comment)
-      		free(comment);
+	free(name);
+	free(comment);
       	return err;
 }
 
 /** 
  * \brief Give a list of device names and associated comments for selected interface
- * \param interface a string identifying interface ("pcm", "ctl", "seq", "rawmidi")
+ * \param iface a string identifying interface ("pcm", "ctl", "seq", "rawmidi")
  * \param list result - a pointer to list
  * \return A non-negative value if successful, otherwise a negative error code.
  *
  * The global configuration files are specified in the environment variable
  * \c ALSA_NAMES_FILE.
  */
-int snd_names_list(const char *interface, snd_devname_t **list)
+int snd_names_list(const char *iface, snd_devname_t **list)
 {
 	char *file;
 	snd_config_t *top;
 	snd_input_t *in;
 	int err;
 	
-	assert(interface);
+	assert(iface);
 	assert(list);
 	*list = NULL;
 	file = getenv(ALSA_NAMES_ENV);
@@ -151,6 +150,7 @@ int snd_names_list(const char *interface, snd_devname_t **list)
 				return -ENOMEM;
 		}
 	}
+	top = NULL;
 	err = snd_config_top(&top);
 	if (err >= 0)
 		err = snd_input_stdio_open(&in, file, "r");
@@ -160,7 +160,7 @@ int snd_names_list(const char *interface, snd_devname_t **list)
 		if (err < 0) {
 			SNDERR("%s may be old or corrupted: consider to remove or fix it", file);
 		} else {
-			err = names_parse(top, interface, list);
+			err = names_parse(top, iface, list);
 			if (err < 0) {
 				snd_names_list_free(*list);
 				*list = NULL;
@@ -169,7 +169,8 @@ int snd_names_list(const char *interface, snd_devname_t **list)
 	} else {
 		SNDERR("cannot access file %s", file);
 	}
-	snd_config_delete(top);
+	if (top)
+		snd_config_delete(top);
 	return err >= 0 ? 0 : err;
 }
 
@@ -185,10 +186,8 @@ void snd_names_list_free(snd_devname_t *list)
 	
 	while (list != NULL) {
 		next = list->next;
-		if (list->name)
-			free(list->name);
-		if (list->comment)
-			free(list->comment);
+		free(list->name);
+		free(list->comment);
 		free(list);
 		list = next;
 	}
