@@ -34,6 +34,7 @@
 #include <string.h>
 #include <fcntl.h>
 #include <ctype.h>
+#include <grp.h>
 #include <sys/ioctl.h>
 #include <sys/mman.h>
 #include <sys/shm.h>
@@ -76,7 +77,7 @@ retryshm:
 	dmix->u.dmix.shmid_sum = shmget(dmix->ipc_key + 1, size,
 					IPC_CREAT | dmix->ipc_perm);
 	err = -errno;
-	if (dmix->u.dmix.shmid_sum < 0){
+	if (dmix->u.dmix.shmid_sum < 0) {
 		if (errno == EINVAL)
 		if ((tmpid = shmget(dmix->ipc_key + 1, 0, dmix->ipc_perm)) != -1)
 		if (!shmctl(tmpid, IPC_STAT, &buf))
@@ -86,10 +87,20 @@ retryshm:
 		    goto retryshm;
 		return err;
 	}
+	if (shmctl(dmix->u.dmix.shmid_sum, IPC_STAT, &buf) < 0) {
+		err = -errno;
+		shm_sum_discard(dmix);
+		return err;
+	}
+	if (dmix->ipc_gid >= 0) {
+		buf.shm_perm.gid = dmix->ipc_gid;
+		shmctl(dmix->u.dmix.shmid_sum, IPC_SET, &buf); 
+	}
 	dmix->u.dmix.sum_buffer = shmat(dmix->u.dmix.shmid_sum, 0, 0);
 	if (dmix->u.dmix.sum_buffer == (void *) -1) {
+		err = -errno;
 		shm_sum_discard(dmix);
-		return -errno;
+		return err;
 	}
 	mlock(dmix->u.dmix.sum_buffer, size);
 	return 0;
@@ -148,7 +159,8 @@ static void mix_areas(snd_pcm_direct_t *dmix,
 	unsigned int chn, dchn, channels;
 	
 	channels = dmix->channels;
-	if (dmix->shmptr->s.format == SND_PCM_FORMAT_S16) {
+	if (dmix->shmptr->s.format == SND_PCM_FORMAT_S16_LE ||
+	    dmix->shmptr->s.format == SND_PCM_FORMAT_S16_BE) {
 		signed short *src;
 		volatile signed short *dst;
 		if (dmix->interleaved) {
@@ -176,7 +188,8 @@ static void mix_areas(snd_pcm_direct_t *dmix,
 			sum = dmix->u.dmix.sum_buffer + channels * dst_ofs + chn;
 			dmix->u.dmix.mix_areas1(size, dst, src, sum, dst_step, src_step, channels * sizeof(signed int));
 		}
-	} else {
+	} else if (dmix->shmptr->s.format == SND_PCM_FORMAT_S32_LE ||
+		   dmix->shmptr->s.format == SND_PCM_FORMAT_S32_BE) {
 		signed int *src;
 		volatile signed int *dst;
 		if (dmix->interleaved) {
@@ -204,6 +217,32 @@ static void mix_areas(snd_pcm_direct_t *dmix,
 			sum = dmix->u.dmix.sum_buffer + channels * dst_ofs + chn;
 			dmix->u.dmix.mix_areas2(size, dst, src, sum, dst_step, src_step, channels * sizeof(signed int));
 		}
+	} else { /* SND_PCM_FORMAT_S24_3LE */
+		unsigned char *src;
+		volatile unsigned char *dst;
+		if (dmix->interleaved) {
+			/*
+			 * process all areas in one loop
+			 * it optimizes the memory accesses for this case
+			 */
+			dmix->u.dmix.mix_areas3(size * channels,
+					((unsigned char *)dst_areas[0].addr) + 3 * dst_ofs * channels,
+					((unsigned char *)src_areas[0].addr) + 3 * src_ofs * channels,
+					dmix->u.dmix.sum_buffer + (dst_ofs * channels),
+					3, 3, sizeof(signed int));
+			return;
+		}
+		for (chn = 0; chn < channels; chn++) {
+			dchn = dmix->bindings ? dmix->bindings[chn] : chn;
+			if (dchn >= dmix->shmptr->s.channels)
+				continue;
+			src_step = src_areas[chn].step / 8;
+			dst_step = dst_areas[dchn].step / 8;
+			src = (unsigned char *)(((char *)src_areas[chn].addr + src_areas[chn].first / 8) + (src_ofs * src_step));
+			dst = (unsigned char *)(((char *)dst_areas[dchn].addr + dst_areas[dchn].first / 8) + (dst_ofs * dst_step));
+			sum = dmix->u.dmix.sum_buffer + channels * dst_ofs + chn;
+			dmix->u.dmix.mix_areas3(size, dst, src, sum, dst_step, src_step, channels * sizeof(signed int));
+		}
 	}
 }
 
@@ -227,49 +266,60 @@ static void mix_areas(snd_pcm_direct_t *dmix,
 static void snd_pcm_dmix_sync_area(snd_pcm_t *pcm)
 {
 	snd_pcm_direct_t *dmix = pcm->private_data;
-	snd_pcm_uframes_t appl_ptr, slave_appl_ptr, slave_bsize;
-	snd_pcm_uframes_t size, slave_hw_ptr;
+	snd_pcm_uframes_t slave_hw_ptr, slave_appl_ptr, slave_size;
+	snd_pcm_uframes_t appl_ptr, size;
 	const snd_pcm_channel_area_t *src_areas, *dst_areas;
 	
 	/* calculate the size to transfer */
+	/* check the available size in the local buffer
+	 * last_appl_ptr keeps the last updated position
+	 */
 	size = dmix->appl_ptr - dmix->last_appl_ptr;
 	if (! size)
 		return;
-	slave_bsize = dmix->shmptr->s.buffer_size;
+	if (size >= pcm->boundary / 2)
+		size = pcm->boundary - size;
+
+	/* check the available size in the slave PCM buffer */
 	slave_hw_ptr = dmix->slave_hw_ptr;
 	/* don't write on the last active period - this area may be cleared
 	 * by the driver during mix operation...
 	 */
-	slave_hw_ptr -= slave_hw_ptr % dmix->shmptr->s.period_size;
-	slave_hw_ptr += slave_bsize;
-	if (dmix->slave_hw_ptr > dmix->slave_appl_ptr)
-		slave_hw_ptr -= dmix->shmptr->s.boundary;
-	if (dmix->slave_appl_ptr + size >= slave_hw_ptr)
-		size = slave_hw_ptr - dmix->slave_appl_ptr;
+	slave_hw_ptr -= slave_hw_ptr % dmix->slave_period_size;
+	slave_hw_ptr += dmix->slave_buffer_size;
+	if (slave_hw_ptr >= dmix->slave_boundary)
+		slave_hw_ptr -= dmix->slave_boundary;
+	if (slave_hw_ptr < dmix->slave_appl_ptr)
+		slave_size = slave_hw_ptr + (dmix->slave_boundary - dmix->slave_appl_ptr);
+	else
+		slave_size = slave_hw_ptr - dmix->slave_appl_ptr;
+	if (slave_size < size)
+		size = slave_size;
 	if (! size)
 		return;
+
 	/* add sample areas here */
 	src_areas = snd_pcm_mmap_areas(pcm);
 	dst_areas = snd_pcm_mmap_areas(dmix->spcm);
 	appl_ptr = dmix->last_appl_ptr % pcm->buffer_size;
 	dmix->last_appl_ptr += size;
 	dmix->last_appl_ptr %= pcm->boundary;
-	slave_appl_ptr = dmix->slave_appl_ptr % slave_bsize;
+	slave_appl_ptr = dmix->slave_appl_ptr % dmix->slave_buffer_size;
 	dmix->slave_appl_ptr += size;
-	dmix->slave_appl_ptr %= dmix->shmptr->s.boundary;
+	dmix->slave_appl_ptr %= dmix->slave_boundary;
 	dmix_down_sem(dmix);
 	for (;;) {
 		snd_pcm_uframes_t transfer = size;
 		if (appl_ptr + transfer > pcm->buffer_size)
 			transfer = pcm->buffer_size - appl_ptr;
-		if (slave_appl_ptr + transfer > slave_bsize)
-			transfer = slave_bsize - slave_appl_ptr;
+		if (slave_appl_ptr + transfer > dmix->slave_buffer_size)
+			transfer = dmix->slave_buffer_size - slave_appl_ptr;
 		mix_areas(dmix, src_areas, dst_areas, appl_ptr, slave_appl_ptr, transfer);
 		size -= transfer;
 		if (! size)
 			break;
 		slave_appl_ptr += transfer;
-		slave_appl_ptr %= slave_bsize;
+		slave_appl_ptr %= dmix->slave_buffer_size;
 		appl_ptr += transfer;
 		appl_ptr %= pcm->buffer_size;
 	}
@@ -288,7 +338,7 @@ static int snd_pcm_dmix_sync_ptr(snd_pcm_t *pcm)
 	switch (snd_pcm_state(dmix->spcm)) {
 	case SND_PCM_STATE_DISCONNECTED:
 		dmix->state = SND_PCM_STATE_DISCONNECTED;
-		return -ENOTTY;
+		return -ENODEV;
 	default:
 		break;
 	}
@@ -304,7 +354,7 @@ static int snd_pcm_dmix_sync_ptr(snd_pcm_t *pcm)
 		/* not really started yet - don't update hw_ptr */
 		return 0;
 	if (diff < 0) {
-		slave_hw_ptr += dmix->shmptr->s.boundary;
+		slave_hw_ptr += dmix->slave_boundary;
 		diff = slave_hw_ptr - old_slave_hw_ptr;
 	}
 	dmix->hw_ptr += diff;
@@ -395,7 +445,7 @@ static int snd_pcm_dmix_delay(snd_pcm_t *pcm, snd_pcm_sframes_t *delayp)
 	case SNDRV_PCM_STATE_XRUN:
 		return -EPIPE;
 	case SNDRV_PCM_STATE_DISCONNECTED:
-		return -ENOTTY;
+		return -ENODEV;
 	default:
 		return -EBADFD;
 	}
@@ -417,7 +467,7 @@ static int snd_pcm_dmix_hwsync(snd_pcm_t *pcm)
 	case SNDRV_PCM_STATE_XRUN:
 		return -EPIPE;
 	case SNDRV_PCM_STATE_DISCONNECTED:
-		return -ENOTTY;
+		return -ENODEV;
 	default:
 		return -EBADFD;
 	}
@@ -428,28 +478,40 @@ static int snd_pcm_dmix_prepare(snd_pcm_t *pcm)
 	snd_pcm_direct_t *dmix = pcm->private_data;
 
 	snd_pcm_direct_check_interleave(dmix, pcm);
-	// assert(pcm->boundary == dmix->shmptr->s.boundary);	/* for sure */
 	dmix->state = SND_PCM_STATE_PREPARED;
 	dmix->appl_ptr = dmix->last_appl_ptr = 0;
 	dmix->hw_ptr = 0;
 	return snd_pcm_direct_set_timer_params(dmix);
 }
 
+static void reset_slave_ptr(snd_pcm_t *pcm, snd_pcm_direct_t *dmix)
+{
+	dmix->slave_appl_ptr = dmix->slave_hw_ptr = *dmix->spcm->hw.ptr;
+	if (pcm->buffer_size > pcm->period_size * 2)
+		return;
+	/* If we have too litte periods, better to align the start position
+	 * to the period boundary so that the interrupt can be handled properly
+	 * at the right time.
+	 */
+	dmix->slave_appl_ptr = ((dmix->slave_appl_ptr + dmix->slave_period_size - 1)
+				/ dmix->slave_period_size) * dmix->slave_period_size;
+}
+
 static int snd_pcm_dmix_reset(snd_pcm_t *pcm)
 {
 	snd_pcm_direct_t *dmix = pcm->private_data;
 	dmix->hw_ptr %= pcm->period_size;
 	dmix->appl_ptr = dmix->last_appl_ptr = dmix->hw_ptr;
-	dmix->slave_appl_ptr = dmix->slave_hw_ptr = *dmix->spcm->hw.ptr;
+	reset_slave_ptr(pcm, dmix);
 	return 0;
 }
 
-static int snd_pcm_dmix_start_timer(snd_pcm_direct_t *dmix)
+static int snd_pcm_dmix_start_timer(snd_pcm_t *pcm, snd_pcm_direct_t *dmix)
 {
 	int err;
 
 	snd_pcm_hwsync(dmix->spcm);
-	dmix->slave_appl_ptr = dmix->slave_hw_ptr = *dmix->spcm->hw.ptr;
+	reset_slave_ptr(pcm, dmix);
 	err = snd_timer_start(dmix->timer);
 	if (err < 0)
 		return err;
@@ -472,7 +534,7 @@ static int snd_pcm_dmix_start(snd_pcm_t *pcm)
 	else if (avail < 0)
 		return 0;
 	else {
-		if ((err = snd_pcm_dmix_start_timer(dmix)) < 0)
+		if ((err = snd_pcm_dmix_start_timer(pcm, dmix)) < 0)
 			return err;
 		snd_pcm_dmix_sync_area(pcm);
 	}
@@ -510,6 +572,12 @@ static int snd_pcm_dmix_drain(snd_pcm_t *pcm)
 			return 0;
 		}
 	}
+
+	if (dmix->state == SND_PCM_STATE_XRUN) {
+		snd_pcm_dmix_drop(pcm);
+		return 0;
+	}
+
 	stop_threshold = pcm->stop_threshold;
 	if (pcm->stop_threshold > pcm->buffer_size)
 		pcm->stop_threshold = pcm->buffer_size;
@@ -559,13 +627,6 @@ static snd_pcm_sframes_t snd_pcm_dmix_forward(snd_pcm_t *pcm, snd_pcm_uframes_t
 	return frames;
 }
 
-static int snd_pcm_dmix_resume(snd_pcm_t *pcm)
-{
-	snd_pcm_direct_t *dmix = pcm->private_data;
-	snd_pcm_resume(dmix->spcm);
-	return 0;
-}
-
 static snd_pcm_sframes_t snd_pcm_dmix_readi(snd_pcm_t *pcm ATTRIBUTE_UNUSED, void *buffer ATTRIBUTE_UNUSED, snd_pcm_uframes_t size ATTRIBUTE_UNUSED)
 {
 	return -ENODEV;
@@ -591,8 +652,7 @@ static int snd_pcm_dmix_close(snd_pcm_t *pcm)
  	shm_sum_discard(dmix);
 	snd_pcm_direct_shm_discard(dmix);
 	snd_pcm_direct_semaphore_up(dmix, DIRECT_IPC_SEM_CLIENT);
-	if (dmix->bindings)
-		free(dmix->bindings);
+	free(dmix->bindings);
 	pcm->private_data = NULL;
 	free(dmix);
 	return 0;
@@ -617,7 +677,7 @@ static snd_pcm_sframes_t snd_pcm_dmix_mmap_commit(snd_pcm_t *pcm,
 		return 0;
 	snd_pcm_mmap_appl_forward(pcm, size);
 	if (dmix->state == STATE_RUN_PENDING) {
-		if ((err = snd_pcm_dmix_start_timer(dmix)) < 0)
+		if ((err = snd_pcm_dmix_start_timer(pcm, dmix)) < 0)
 			return err;
 	} else if (dmix->state == SND_PCM_STATE_RUNNING ||
 		   dmix->state == SND_PCM_STATE_DRAINING)
@@ -659,7 +719,7 @@ static void snd_pcm_dmix_dump(snd_pcm_t *pcm, snd_output_t *out)
 
 	snd_output_printf(out, "Direct Stream Mixing PCM\n");
 	if (pcm->setup) {
-		snd_output_printf(out, "\nIts setup is:\n");
+		snd_output_printf(out, "Its setup is:\n");
 		snd_pcm_dump_setup(pcm, out);
 	}
 	if (dmix->spcm)
@@ -694,7 +754,7 @@ static snd_pcm_fast_ops_t snd_pcm_dmix_fast_ops = {
 	.pause = snd_pcm_dmix_pause,
 	.rewind = snd_pcm_dmix_rewind,
 	.forward = snd_pcm_dmix_forward,
-	.resume = snd_pcm_dmix_resume,
+	.resume = snd_pcm_direct_resume,
 	.link_fd = NULL,
 	.link = NULL,
 	.unlink = NULL,
@@ -713,11 +773,8 @@ static snd_pcm_fast_ops_t snd_pcm_dmix_fast_ops = {
  * \brief Creates a new dmix PCM
  * \param pcmp Returns created PCM handle
  * \param name Name of PCM
- * \param ipc_key IPC key for semaphore and shared memory
- * \param ipc_perm IPC permissions for semaphore and shared memory
+ * \param opts Direct PCM configurations
  * \param params Parameters for slave
- * \param bindings Channel bindings
- * \param slowptr Slow but more precise pointer updates
  * \param root Configuration root
  * \param sconf Slave configuration
  * \param stream PCM Direction (stream)
@@ -728,10 +785,8 @@ static snd_pcm_fast_ops_t snd_pcm_dmix_fast_ops = {
  *          changed in future.
  */
 int snd_pcm_dmix_open(snd_pcm_t **pcmp, const char *name,
-		      key_t ipc_key, mode_t ipc_perm,
+		      struct snd_pcm_direct_open_conf *opts,
 		      struct slave_params *params,
-		      snd_config_t *bindings,
-		      int slowptr,
 		      snd_config_t *root, snd_config_t *sconf,
 		      snd_pcm_stream_t stream, int mode)
 {
@@ -753,12 +808,13 @@ int snd_pcm_dmix_open(snd_pcm_t **pcmp, const char *name,
 		goto _err_nosem;
 	}
 	
-	ret = snd_pcm_direct_parse_bindings(dmix, bindings);
+	ret = snd_pcm_direct_parse_bindings(dmix, opts->bindings);
 	if (ret < 0)
 		goto _err_nosem;
 	
-	dmix->ipc_key = ipc_key;
-	dmix->ipc_perm = ipc_perm;
+	dmix->ipc_key = opts->ipc_key;
+	dmix->ipc_perm = opts->ipc_perm;
+	dmix->ipc_gid = opts->ipc_gid;
 	dmix->semid = -1;
 	dmix->shmid = -1;
 
@@ -793,11 +849,15 @@ int snd_pcm_dmix_open(snd_pcm_t **pcmp, const char *name,
 	pcm->fast_ops = &snd_pcm_dmix_fast_ops;
 	pcm->private_data = dmix;
 	dmix->state = SND_PCM_STATE_OPEN;
-	dmix->slowptr = slowptr;
+	dmix->slowptr = opts->slowptr;
+	dmix->max_periods = opts->max_periods;
 	dmix->sync_ptr = snd_pcm_dmix_sync_ptr;
 
 	if (first_instance) {
-		ret = snd_pcm_open_slave(&spcm, root, sconf, stream, mode | SND_PCM_NONBLOCK);
+		/* recursion is already checked in
+		   snd_pcm_direct_get_slave_ipc_offset() */
+		ret = snd_pcm_open_slave(&spcm, root, sconf, stream,
+					 mode | SND_PCM_NONBLOCK, NULL);
 		if (ret < 0) {
 			SNDERR("unable to open slave");
 			goto _err;
@@ -836,25 +896,9 @@ int snd_pcm_dmix_open(snd_pcm_t **pcmp, const char *name,
 		}
 			
 		snd_pcm_direct_semaphore_down(dmix, DIRECT_IPC_SEM_CLIENT);
-		ret = snd_pcm_hw_open_fd(&spcm, "dmix_client", dmix->hw_fd, 0, 0);
-		if (ret < 0) {
-			SNDERR("unable to open hardware");
+		ret = snd_pcm_direct_open_secondary_client(&spcm, dmix, "dmix_client");
+		if (ret < 0)
 			goto _err;
-		}
-		
-		spcm->donot_close = 1;
-		spcm->setup = 1;
-		spcm->buffer_size = dmix->shmptr->s.buffer_size;
-		spcm->sample_bits = dmix->shmptr->s.sample_bits;
-		spcm->channels = dmix->shmptr->s.channels;
-		spcm->format = dmix->shmptr->s.format;
-		spcm->boundary = dmix->shmptr->s.boundary;
-		spcm->info = dmix->shmptr->s.info;
-		ret = snd_pcm_mmap(spcm);
-		if (ret < 0) {
-			SNDERR("unable to mmap channels");
-			goto _err;
-		}
 		dmix->spcm = spcm;
 	}
 
@@ -904,8 +948,7 @@ int snd_pcm_dmix_open(snd_pcm_t **pcmp, const char *name,
 		snd_pcm_direct_semaphore_up(dmix, DIRECT_IPC_SEM_CLIENT);
  _err_nosem:
 	if (dmix) {
-		if (dmix->bindings)
-			free(dmix->bindings);
+		free(dmix->bindings);
 		free(dmix);
 	}
 	if (pcm)
@@ -1044,81 +1087,16 @@ int _snd_pcm_dmix_open(snd_pcm_t **pcmp, const char *name,
 		       snd_config_t *root, snd_config_t *conf,
 		       snd_pcm_stream_t stream, int mode)
 {
-	snd_config_iterator_t i, next;
-	snd_config_t *slave = NULL, *bindings = NULL, *sconf;
+	snd_config_t *sconf;
 	struct slave_params params;
-	int bsize, psize, ipc_key_add_uid = 0, slowptr = 0;
-	key_t ipc_key = 0;
-	mode_t ipc_perm = 0600;
+	struct snd_pcm_direct_open_conf dopen;
+	int bsize, psize;
 	int err;
-	snd_config_for_each(i, next, conf) {
-		snd_config_t *n = snd_config_iterator_entry(i);
-		const char *id;
-		if (snd_config_get_id(n, &id) < 0)
-			continue;
-		if (snd_pcm_conf_generic_id(id))
-			continue;
-		if (strcmp(id, "ipc_key") == 0) {
-			long key;
-			err = snd_config_get_integer(n, &key);
-			if (err < 0) {
-				SNDERR("The field ipc_key must be an integer type");
-				return err;
-			}
-			ipc_key = key;
-			continue;
-		}
-		if (strcmp(id, "ipc_perm") == 0) {
-			char *perm;
-			char *endp;
-			err = snd_config_get_ascii(n, &perm);
-			if (err < 0) {
-				SNDERR("The field ipc_perm must be a valid file permission");
-				return err;
-			}
-			if (isdigit(*perm) == 0) {
-				SNDERR("The field ipc_perm must be a valid file permission");
-				return -EINVAL;
-			}
-			ipc_perm = strtol(perm, &endp, 8);
-			continue;
-		}
-		if (strcmp(id, "ipc_key_add_uid") == 0) {
-			if ((err = snd_config_get_bool(n)) < 0) {
-				SNDERR("The field ipc_key_add_uid must be a boolean type");
-				return err;
-			}
-			ipc_key_add_uid = err;
-			continue;
-		}
-		if (strcmp(id, "slave") == 0) {
-			slave = n;
-			continue;
-		}
-		if (strcmp(id, "bindings") == 0) {
-			bindings = n;
-			continue;
-		}
-		if (strcmp(id, "slowptr") == 0) {
-			err = snd_config_get_bool(n);
-			if (err < 0)
-				return err;
-			slowptr = err;
-			continue;
-		}
-		SNDERR("Unknown field %s", id);
-		return -EINVAL;
-	}
-	if (!slave) {
-		SNDERR("slave is not defined");
-		return -EINVAL;
-	}
-	if (ipc_key_add_uid)
-		ipc_key += getuid();
-	if (!ipc_key) {
-		SNDERR("Unique IPC key is not defined");
-		return -EINVAL;
-	}
+
+	err = snd_pcm_direct_parse_open_conf(root, conf, stream, &dopen);
+	if (err < 0)
+		return err;
+
 	/* the default settings, it might be invalid for some hardware */
 	params.format = SND_PCM_FORMAT_S16;
 	params.rate = 48000;
@@ -1128,7 +1106,7 @@ int _snd_pcm_dmix_open(snd_pcm_t **pcmp, const char *name,
 	bsize = psize = -1;
 	params.periods = 3;
 
-	err = snd_pcm_slave_conf(root, slave, &sconf, 8,
+	err = snd_pcm_slave_conf(root, dopen.slave, &sconf, 8,
 				 SND_PCM_HW_PARAM_FORMAT, 0, &params.format,
 				 SND_PCM_HW_PARAM_RATE, 0, &params.rate,
 				 SND_PCM_HW_PARAM_CHANNELS, 0, &params.channels,
@@ -1145,9 +1123,8 @@ int _snd_pcm_dmix_open(snd_pcm_t **pcmp, const char *name,
 		params.period_time = 125000;    /* 0.125 seconds */
 
 	/* sorry, limited features */
-        if (params.format != SND_PCM_FORMAT_S16 &&
-            params.format != SND_PCM_FORMAT_S32) {
-		SNDERR("invalid format, specify s16 or s32");
+	if (! (dmix_supported_format & (1ULL << params.format))) {
+		SNDERR("Unsupported format");
 		snd_config_delete(sconf);
 		return -EINVAL;
 	}
@@ -1155,7 +1132,8 @@ int _snd_pcm_dmix_open(snd_pcm_t **pcmp, const char *name,
 	params.period_size = psize;
 	params.buffer_size = bsize;
 
-	err = snd_pcm_dmix_open(pcmp, name, ipc_key, ipc_perm, &params, bindings, slowptr, root, sconf, stream, mode);
+	err = snd_pcm_dmix_open(pcmp, name, &dopen, &params,
+				root, sconf, stream, mode);
 	if (err < 0)
 		snd_config_delete(sconf);
 	return err;
