@@ -34,6 +34,7 @@
 #include <string.h>
 #include <fcntl.h>
 #include <ctype.h>
+#include <grp.h>
 #include <sys/ioctl.h>
 #include <sys/mman.h>
 #include <sys/shm.h>
@@ -92,14 +93,15 @@ static void snd_pcm_dsnoop_sync_area(snd_pcm_t *pcm, snd_pcm_uframes_t slave_hw_
 	dst_areas = snd_pcm_mmap_areas(pcm);
 	src_areas = snd_pcm_mmap_areas(dsnoop->spcm);
 	hw_ptr %= pcm->buffer_size;
-	slave_hw_ptr %= dsnoop->shmptr->s.buffer_size;
+	slave_hw_ptr %= dsnoop->slave_buffer_size;
 	while (size > 0) {
 		transfer = hw_ptr + size > pcm->buffer_size ? pcm->buffer_size - hw_ptr : size;
-		transfer = slave_hw_ptr + transfer > dsnoop->shmptr->s.buffer_size ? dsnoop->shmptr->s.buffer_size - slave_hw_ptr : transfer;
+		transfer = slave_hw_ptr + transfer > dsnoop->slave_buffer_size ?
+			dsnoop->slave_buffer_size - slave_hw_ptr : transfer;
 		size -= transfer;
 		snoop_areas(dsnoop, src_areas, dst_areas, slave_hw_ptr, hw_ptr, transfer);
 		slave_hw_ptr += transfer;
-	 	slave_hw_ptr %= dsnoop->shmptr->s.buffer_size;
+	 	slave_hw_ptr %= dsnoop->slave_buffer_size;
 		hw_ptr += transfer;
 		hw_ptr %= pcm->buffer_size;
 	}
@@ -117,7 +119,7 @@ static int snd_pcm_dsnoop_sync_ptr(snd_pcm_t *pcm)
 	switch (snd_pcm_state(dsnoop->spcm)) {
 	case SND_PCM_STATE_DISCONNECTED:
 		dsnoop->state = SNDRV_PCM_STATE_DISCONNECTED;
-		return -ENOTTY;
+		return -ENODEV;
 	default:
 		break;
 	}
@@ -129,7 +131,7 @@ static int snd_pcm_dsnoop_sync_ptr(snd_pcm_t *pcm)
 	if (diff == 0)		/* fast path */
 		return 0;
 	if (diff < 0) {
-		slave_hw_ptr += dsnoop->shmptr->s.boundary;
+		slave_hw_ptr += dsnoop->slave_boundary;
 		diff = slave_hw_ptr - old_slave_hw_ptr;
 	}
 	snd_pcm_dsnoop_sync_area(pcm, old_slave_hw_ptr, diff);
@@ -188,7 +190,7 @@ static snd_pcm_state_t snd_pcm_dsnoop_state(snd_pcm_t *pcm)
 		return SND_PCM_STATE_SUSPENDED;
 	case SND_PCM_STATE_DISCONNECTED:
 		dsnoop->state = SNDRV_PCM_STATE_DISCONNECTED;
-		return -ENOTTY;
+		return -ENODEV;
 	default:
 		break;
 	}
@@ -213,7 +215,7 @@ static int snd_pcm_dsnoop_delay(snd_pcm_t *pcm, snd_pcm_sframes_t *delayp)
 	case SNDRV_PCM_STATE_XRUN:
 		return -EPIPE;
 	case SNDRV_PCM_STATE_DISCONNECTED:
-		return -ENOTTY;
+		return -ENODEV;
 	default:
 		return -EBADFD;
 	}
@@ -233,7 +235,7 @@ static int snd_pcm_dsnoop_hwsync(snd_pcm_t *pcm)
 	case SNDRV_PCM_STATE_XRUN:
 		return -EPIPE;
 	case SNDRV_PCM_STATE_DISCONNECTED:
-		return -ENOTTY;
+		return -ENODEV;
 	default:
 		return -EBADFD;
 	}
@@ -244,7 +246,6 @@ static int snd_pcm_dsnoop_prepare(snd_pcm_t *pcm)
 	snd_pcm_direct_t *dsnoop = pcm->private_data;
 
 	snd_pcm_direct_check_interleave(dsnoop, pcm);
-	// assert(pcm->boundary == dsnoop->shmptr->s.boundary);	/* for sure */
 	dsnoop->state = SND_PCM_STATE_PREPARED;
 	dsnoop->appl_ptr = 0;
 	dsnoop->hw_ptr = 0;
@@ -337,13 +338,6 @@ static snd_pcm_sframes_t snd_pcm_dsnoop_forward(snd_pcm_t *pcm, snd_pcm_uframes_
 	return frames;
 }
 
-static int snd_pcm_dsnoop_resume(snd_pcm_t *pcm)
-{
-	snd_pcm_direct_t *dsnoop = pcm->private_data;
-	snd_pcm_resume(dsnoop->spcm);
-	return 0;
-}
-
 static snd_pcm_sframes_t snd_pcm_dsnoop_writei(snd_pcm_t *pcm ATTRIBUTE_UNUSED, const void *buffer ATTRIBUTE_UNUSED, snd_pcm_uframes_t size ATTRIBUTE_UNUSED)
 {
 	return -ENODEV;
@@ -368,8 +362,7 @@ static int snd_pcm_dsnoop_close(snd_pcm_t *pcm)
  		snd_pcm_direct_client_discard(dsnoop);
 	snd_pcm_direct_shm_discard(dsnoop);
 	snd_pcm_direct_semaphore_up(dsnoop, DIRECT_IPC_SEM_CLIENT);
-	if (dsnoop->bindings)
-		free(dsnoop->bindings);
+	free(dsnoop->bindings);
 	pcm->private_data = NULL;
 	free(dsnoop);
 	return 0;
@@ -402,7 +395,7 @@ static snd_pcm_sframes_t snd_pcm_dsnoop_mmap_commit(snd_pcm_t *pcm,
 	return size;
 }
 
-static snd_pcm_sframes_t snd_pcm_dsnoop_avail_update(snd_pcm_t *pcm ATTRIBUTE_UNUSED)
+static snd_pcm_sframes_t snd_pcm_dsnoop_avail_update(snd_pcm_t *pcm)
 {
 	snd_pcm_direct_t *dsnoop = pcm->private_data;
 	int err;
@@ -419,9 +412,9 @@ static void snd_pcm_dsnoop_dump(snd_pcm_t *pcm, snd_output_t *out)
 {
 	snd_pcm_direct_t *dsnoop = pcm->private_data;
 
-	snd_output_printf(out, "Direct Stream Mixing PCM\n");
+	snd_output_printf(out, "Direct Snoop PCM\n");
 	if (pcm->setup) {
-		snd_output_printf(out, "\nIts setup is:\n");
+		snd_output_printf(out, "Its setup is:\n");
 		snd_pcm_dump_setup(pcm, out);
 	}
 	if (dsnoop->spcm)
@@ -456,7 +449,7 @@ static snd_pcm_fast_ops_t snd_pcm_dsnoop_fast_ops = {
 	.pause = snd_pcm_dsnoop_pause,
 	.rewind = snd_pcm_dsnoop_rewind,
 	.forward = snd_pcm_dsnoop_forward,
-	.resume = snd_pcm_dsnoop_resume,
+	.resume = snd_pcm_direct_resume,
 	.link_fd = NULL,
 	.link = NULL,
 	.unlink = NULL,
@@ -475,11 +468,8 @@ static snd_pcm_fast_ops_t snd_pcm_dsnoop_fast_ops = {
  * \brief Creates a new dsnoop PCM
  * \param pcmp Returns created PCM handle
  * \param name Name of PCM
- * \param ipc_key IPC key for semaphore and shared memory
- * \param ipc_perm IPC permissions for semaphore and shared memory
+ * \param opts Direct PCM configurations
  * \param params Parameters for slave
- * \param bindings Channel bindings
- * \param slowptr Slow but more precise pointer updates
  * \param root Configuration root
  * \param sconf Slave configuration
  * \param stream PCM Direction (stream)
@@ -490,10 +480,8 @@ static snd_pcm_fast_ops_t snd_pcm_dsnoop_fast_ops = {
  *          changed in future.
  */
 int snd_pcm_dsnoop_open(snd_pcm_t **pcmp, const char *name,
-			key_t ipc_key, mode_t ipc_perm,
+			struct snd_pcm_direct_open_conf *opts,
 			struct slave_params *params,
-			snd_config_t *bindings,
-			int slowptr,
 			snd_config_t *root, snd_config_t *sconf,
 			snd_pcm_stream_t stream, int mode)
 {
@@ -514,12 +502,13 @@ int snd_pcm_dsnoop_open(snd_pcm_t **pcmp, const char *name,
 		goto _err_nosem;
 	}
 	
-	ret = snd_pcm_direct_parse_bindings(dsnoop, bindings);
+	ret = snd_pcm_direct_parse_bindings(dsnoop, opts->bindings);
 	if (ret < 0)
 		goto _err_nosem;
 	
-	dsnoop->ipc_key = ipc_key;
-	dsnoop->ipc_perm = ipc_perm;
+	dsnoop->ipc_key = opts->ipc_key;
+	dsnoop->ipc_perm = opts->ipc_perm;
+	dsnoop->ipc_gid = opts->ipc_gid;
 	dsnoop->semid = -1;
 	dsnoop->shmid = -1;
 
@@ -554,11 +543,15 @@ int snd_pcm_dsnoop_open(snd_pcm_t **pcmp, const char *name,
 	pcm->fast_ops = &snd_pcm_dsnoop_fast_ops;
 	pcm->private_data = dsnoop;
 	dsnoop->state = SND_PCM_STATE_OPEN;
-	dsnoop->slowptr = slowptr;
+	dsnoop->slowptr = opts->slowptr;
+	dsnoop->max_periods = opts->max_periods;
 	dsnoop->sync_ptr = snd_pcm_dsnoop_sync_ptr;
 
 	if (first_instance) {
-		ret = snd_pcm_open_slave(&spcm, root, sconf, stream, mode | SND_PCM_NONBLOCK);
+		/* recursion is already checked in
+		   snd_pcm_direct_get_slave_ipc_offset() */
+		ret = snd_pcm_open_slave(&spcm, root, sconf, stream,
+					 mode | SND_PCM_NONBLOCK, NULL);
 		if (ret < 0) {
 			SNDERR("unable to open slave");
 			goto _err;
@@ -594,25 +587,10 @@ int snd_pcm_dsnoop_open(snd_pcm_t **pcmp, const char *name,
 		}
 			
 		snd_pcm_direct_semaphore_down(dsnoop, DIRECT_IPC_SEM_CLIENT);
-		ret = snd_pcm_hw_open_fd(&spcm, "dsnoop_client", dsnoop->hw_fd, 0, 0);
-		if (ret < 0) {
-			SNDERR("unable to open hardware");
+
+		ret = snd_pcm_direct_open_secondary_client(&spcm, dsnoop, "dsnoop_client");
+		if (ret < 0)
 			goto _err;
-		}
-		
-		spcm->donot_close = 1;
-		spcm->setup = 1;
-		spcm->buffer_size = dsnoop->shmptr->s.buffer_size;
-		spcm->sample_bits = dsnoop->shmptr->s.sample_bits;
-		spcm->channels = dsnoop->shmptr->s.channels;
-		spcm->format = dsnoop->shmptr->s.format;
-		spcm->boundary = dsnoop->shmptr->s.boundary;
-		spcm->info = dsnoop->shmptr->s.info;
-		ret = snd_pcm_mmap(spcm);
-		if (ret < 0) {
-			SNDERR("unable to mmap channels");
-			goto _err;
-		}
 		dsnoop->spcm = spcm;
 	}
 
@@ -652,8 +630,7 @@ int snd_pcm_dsnoop_open(snd_pcm_t **pcmp, const char *name,
 		snd_pcm_direct_semaphore_up(dsnoop, DIRECT_IPC_SEM_CLIENT);
  _err_nosem:
 	if (dsnoop) {
-		if (dsnoop->bindings)
-			free(dsnoop->bindings);
+		free(dsnoop->bindings);
 		free(dsnoop);
 	}
 	if (pcm)
@@ -726,85 +703,16 @@ int _snd_pcm_dsnoop_open(snd_pcm_t **pcmp, const char *name,
 		       snd_config_t *root, snd_config_t *conf,
 		       snd_pcm_stream_t stream, int mode)
 {
-	snd_config_iterator_t i, next;
-	snd_config_t *slave = NULL, *bindings = NULL, *sconf;
+	snd_config_t *sconf;
 	struct slave_params params;
-	int bsize, psize, ipc_key_add_uid = 0, slowptr = 0;
-	key_t ipc_key = 0;
-	mode_t ipc_perm = 0600;
+	struct snd_pcm_direct_open_conf dopen;
+	int bsize, psize;
 	int err;
 
-	snd_config_for_each(i, next, conf) {
-		snd_config_t *n = snd_config_iterator_entry(i);
-		const char *id;
-		if (snd_config_get_id(n, &id) < 0)
-			continue;
-		if (snd_pcm_conf_generic_id(id))
-			continue;
-		if (strcmp(id, "ipc_key") == 0) {
-			long key;
-			err = snd_config_get_integer(n, &key);
-			if (err < 0) {
-				SNDERR("The field ipc_key must be an integer type");
-				return err;
-			}
-			ipc_key = key;
-			continue;
-		}
-		if (strcmp(id, "ipc_perm") == 0) {
-			char *perm;
-			char *endp;
-			err = snd_config_get_ascii(n, &perm);
-			if (err < 0) {
-				SNDERR("The field ipc_perm must be a valid file permission");
-				return err;
-			}
-			if (isdigit(*perm) == 0) {
-				SNDERR("The field ipc_perm must be a valid file permission");
-				return -EINVAL;
-			}
-			ipc_perm = strtol(perm, &endp, 8);
-			continue;
-		}
-		if (strcmp(id, "ipc_key_add_uid") == 0) {
-			err = snd_config_get_bool(n);
-			if (err < 0) {
-				SNDERR("The field ipc_key_add_uid must be a boolean type");
-				return err;
-			}
-			ipc_key_add_uid = err;
-			continue;
-		}
-		if (strcmp(id, "slave") == 0) {
-			slave = n;
-			continue;
-		}
-		if (strcmp(id, "bindings") == 0) {
-			bindings = n;
-			continue;
-		}
-		if (strcmp(id, "slowptr") == 0) {
-			err = snd_config_get_bool(n);
-			if (err < 0) {
-				SNDERR("The field slowptr must be a boolean type");
-				return err;
-			}
-			slowptr = err;
-			continue;
-		}
-		SNDERR("Unknown field %s", id);
-		return -EINVAL;
-	}
-	if (!slave) {
-		SNDERR("slave is not defined");
-		return -EINVAL;
-	}
-	if (ipc_key_add_uid)
-		ipc_key += getuid();
-	if (!ipc_key) {
-		SNDERR("Unique IPC key is not defined");
-		return -EINVAL;
-	}
+	err = snd_pcm_direct_parse_open_conf(root, conf, stream, &dopen);
+	if (err < 0)
+		return err;
+
 	/* the default settings, it might be invalid for some hardware */
 	params.format = SND_PCM_FORMAT_S16;
 	params.rate = 48000;
@@ -813,7 +721,7 @@ int _snd_pcm_dsnoop_open(snd_pcm_t **pcmp, const char *name,
 	params.buffer_time = -1;
 	bsize = psize = -1;
 	params.periods = 3;
-	err = snd_pcm_slave_conf(root, slave, &sconf, 8,
+	err = snd_pcm_slave_conf(root, dopen.slave, &sconf, 8,
 				 SND_PCM_HW_PARAM_FORMAT, 0, &params.format,
 				 SND_PCM_HW_PARAM_RATE, 0, &params.rate,
 				 SND_PCM_HW_PARAM_CHANNELS, 0, &params.channels,
@@ -831,7 +739,9 @@ int _snd_pcm_dsnoop_open(snd_pcm_t **pcmp, const char *name,
 
 	params.period_size = psize;
 	params.buffer_size = bsize;
-	err = snd_pcm_dsnoop_open(pcmp, name, ipc_key, ipc_perm, &params, bindings, slowptr, root, sconf, stream, mode);
+
+	err = snd_pcm_dsnoop_open(pcmp, name, &dopen, &params,
+				  root, sconf, stream, mode);
 	if (err < 0)
 		snd_config_delete(sconf);
 	return err;
