@@ -153,8 +153,7 @@ static int snd_pcm_meter_update_scope(snd_pcm_t *pcm)
 
 static int snd_pcm_scope_remove(snd_pcm_scope_t *scope)
 {
-	if (scope->name)
-		free((void *)scope->name);
+	free(scope->name);
 	scope->ops->close(scope);
 	list_del(&scope->list);
 	free(scope);
@@ -480,12 +479,10 @@ static int snd_pcm_meter_hw_free(snd_pcm_t *pcm)
 	pthread_mutex_unlock(&meter->running_mutex);
 	err = pthread_join(meter->thread, 0);
 	assert(err == 0);
-	if (meter->buf) {
-		free(meter->buf);
-		free(meter->buf_areas);
-		meter->buf = 0;
-		meter->buf_areas = 0;
-	}
+	free(meter->buf);
+	free(meter->buf_areas);
+	meter->buf = NULL;
+	meter->buf_areas = NULL;
 	return snd_pcm_hw_free(meter->gen.slave);
 }
 
@@ -575,6 +572,7 @@ int snd_pcm_meter_open(snd_pcm_t **pcmp, const char *name, unsigned int frequenc
 		return err;
 	}
 	pcm->mmap_rw = 1;
+	pcm->mmap_shadow = 1;
 	pcm->ops = &snd_pcm_meter_ops;
 	pcm->fast_ops = &snd_pcm_meter_fast_ops;
 	pcm->private_data = meter;
@@ -778,7 +776,7 @@ int _snd_pcm_meter_open(snd_pcm_t **pcmp, const char *name,
 	err = snd_pcm_slave_conf(root, slave, &sconf, 0);
 	if (err < 0)
 		return err;
-	err = snd_pcm_open_slave(&spcm, root, sconf, stream, mode);
+	err = snd_pcm_open_slave(&spcm, root, sconf, stream, mode, conf);
 	snd_config_delete(sconf);
 	if (err < 0)
 		return err;
@@ -1027,15 +1025,13 @@ static int s16_enable(snd_pcm_scope_t *scope)
 	}
 	s16->buf = malloc(meter->buf_size * 2 * spcm->channels);
 	if (!s16->buf) {
-		if (s16->adpcm_states)
-			free(s16->adpcm_states);
+		free(s16->adpcm_states);
 		return -ENOMEM;
 	}
 	a = calloc(spcm->channels, sizeof(*a));
 	if (!a) {
 		free(s16->buf);
-		if (s16->adpcm_states)
-			free(s16->adpcm_states);
+		free(s16->adpcm_states);
 		return -ENOMEM;
 	}
 	s16->buf_areas = a;
@@ -1050,10 +1046,8 @@ static int s16_enable(snd_pcm_scope_t *scope)
 static void s16_disable(snd_pcm_scope_t *scope)
 {
 	snd_pcm_scope_s16_t *s16 = scope->private_data;
-	if (s16->adpcm_states) {
-		free(s16->adpcm_states);
-		s16->adpcm_states = NULL;
-	}
+	free(s16->adpcm_states);
+	s16->adpcm_states = NULL;
 	free(s16->buf);
 	s16->buf = NULL;
 	free(s16->buf_areas);
