@@ -341,14 +341,12 @@ int snd_rawmidi_close(snd_rawmidi_t *rawmidi)
 {
 	int err;
   	assert(rawmidi);
-	if ((err = rawmidi->ops->close(rawmidi)) < 0)
-		return err;
-	if (rawmidi->name)
-		free(rawmidi->name);
+	err = rawmidi->ops->close(rawmidi);
+	free(rawmidi->name);
 	if (rawmidi->dl_handle)
 		snd_dlclose(rawmidi->dl_handle);
 	free(rawmidi);
-	return 0;
+	return err;
 }
 
 /**
@@ -574,7 +572,7 @@ unsigned int snd_rawmidi_info_get_flags(const snd_rawmidi_info_t *info)
 const char *snd_rawmidi_info_get_id(const snd_rawmidi_info_t *info)
 {
 	assert(info);
-	return info->id;
+	return (const char *)info->id;
 }
 
 /**
@@ -585,7 +583,7 @@ const char *snd_rawmidi_info_get_id(const snd_rawmidi_info_t *info)
 const char *snd_rawmidi_info_get_name(const snd_rawmidi_info_t *info)
 {
 	assert(info);
-	return info->name;
+	return (const char *)info->name;
 }
 
 /**
@@ -596,7 +594,7 @@ const char *snd_rawmidi_info_get_name(const snd_rawmidi_info_t *info)
 const char *snd_rawmidi_info_get_subdevice_name(const snd_rawmidi_info_t *info)
 {
 	assert(info);
-	return info->subname;
+	return (const char *)info->subname;
 }
 
 /**
