@@ -33,8 +33,8 @@
 const char *_snd_module_timer_hw = "";
 #endif
 
-#define SNDRV_FILE_TIMER		"/dev/snd/timer"
-#define SNDRV_TIMER_VERSION_MAX	SNDRV_PROTOCOL_VERSION(2, 0, 1)
+#define SNDRV_FILE_TIMER		ALSA_DEVICE_DIRECTORY "timer"
+#define SNDRV_TIMER_VERSION_MAX	SNDRV_PROTOCOL_VERSION(2, 0, 5)
 
 #define SNDRV_TIMER_IOCTL_STATUS_OLD	_IOW('T', 0x14, struct sndrv_timer_status)
 
@@ -258,10 +258,15 @@ int snd_timer_hw_open(snd_timer_t **handle, const char *name, int dev_class, int
 	}
 	if (mode & SND_TIMER_OPEN_TREAD) {
 		int arg = 1;
+		if (ver < SNDRV_PROTOCOL_VERSION(2, 0, 3)) {
+			ret = -ENOTTY;
+			goto __no_tread;
+		}
 		if (ioctl(fd, SNDRV_TIMER_IOCTL_TREAD, &arg) < 0) {
 			ret = -errno;
 			close(fd);
-			SNDERR("extended read is not supported (SNDRV_TIMER_IOCTL_TREAD)");
+		      __no_tread:
+			SNDMSG("extended read is not supported (SNDRV_TIMER_IOCTL_TREAD)");
 			return ret;
 		}
 	}
