@@ -196,12 +196,10 @@ int snd_timer_query_close(snd_timer_query_t *timer)
 {
 	int err;
   	assert(timer);
-	if ((err = timer->ops->close(timer)) < 0)
-		return err;
-	if (timer->name)
-		free(timer->name);
+	err = timer->ops->close(timer);
+	free(timer->name);
 	free(timer);
-	return 0;
+	return err;
 }
 
 /**
@@ -319,7 +317,7 @@ int snd_timer_ginfo_get_card(snd_timer_ginfo_t *obj)
  */
 char *snd_timer_ginfo_get_id(snd_timer_ginfo_t *obj)
 {
-	return obj->id;
+	return (char *)obj->id;
 }
 
 /**
@@ -329,7 +327,7 @@ char *snd_timer_ginfo_get_id(snd_timer_ginfo_t *obj)
  */
 char *snd_timer_ginfo_get_name(snd_timer_ginfo_t *obj)
 {
-	return obj->name;
+	return (char *)obj->name;
 }
 
 /**
