@@ -12,19 +12,19 @@
 #include <sys/time.h>
 #include <math.h>
 
-char *device = "plughw:0,0";			/* playback device */
-snd_pcm_format_t format = SND_PCM_FORMAT_S16;	/* sample format */
-unsigned int rate = 44100;			/* stream rate */
-unsigned int channels = 1;			/* count of channels */
-unsigned int buffer_time = 500000;		/* ring buffer length in us */
-unsigned int period_time = 100000;		/* period time in us */
-double freq = 440;				/* sinusoidal wave frequency in Hz */
-int verbose = 0;				/* verbose flag */
-int resample = 1;				/* enable alsa-lib resampling */
+static char *device = "plughw:0,0";			/* playback device */
+static snd_pcm_format_t format = SND_PCM_FORMAT_S16;	/* sample format */
+static unsigned int rate = 44100;			/* stream rate */
+static unsigned int channels = 1;			/* count of channels */
+static unsigned int buffer_time = 500000;		/* ring buffer length in us */
+static unsigned int period_time = 100000;		/* period time in us */
+static double freq = 440;				/* sinusoidal wave frequency in Hz */
+static int verbose = 0;				/* verbose flag */
+static int resample = 1;				/* enable alsa-lib resampling */
 
-snd_pcm_sframes_t buffer_size;
-snd_pcm_sframes_t period_size;
-snd_output_t *output = NULL;
+static snd_pcm_sframes_t buffer_size;
+static snd_pcm_sframes_t period_size;
+static snd_output_t *output = NULL;
 
 static void generate_sine(const snd_pcm_channel_area_t *areas, 
 			  snd_pcm_uframes_t offset,
@@ -34,10 +34,12 @@ static void generate_sine(const snd_pcm_channel_area_t *areas,
 	double phase = *_phase;
 	double step = max_phase*freq/(double)rate;
 	double res;
-	signed short *samples[channels];
+	unsigned char *samples[channels], *tmp;
 	int steps[channels];
-	unsigned int chn;
+	unsigned int chn, byte;
 	int ires;
+	unsigned int maxval = (1 << (snd_pcm_format_width(format) - 1)) - 1;
+	int bps = snd_pcm_format_width(format) / 8;  /* bytes per sample */
 	
 	/* verify and prepare the contents of areas */
 	for (chn = 0; chn < channels; chn++) {
@@ -45,20 +47,22 @@ static void generate_sine(const snd_pcm_channel_area_t *areas,
 			printf("areas[%i].first == %i, aborting...\n", chn, areas[chn].first);
 			exit(EXIT_FAILURE);
 		}
-		samples[chn] = (signed short *)(((unsigned char *)areas[chn].addr) + (areas[chn].first / 8));
+		samples[chn] = /*(signed short *)*/(((unsigned char *)areas[chn].addr) + (areas[chn].first / 8));
 		if ((areas[chn].step % 16) != 0) {
 			printf("areas[%i].step == %i, aborting...\n", chn, areas[chn].step);
 			exit(EXIT_FAILURE);
 		}
-		steps[chn] = areas[chn].step / 16;
+		steps[chn] = areas[chn].step / 8;
 		samples[chn] += offset * steps[chn];
 	}
 	/* fill the channel areas */
 	while (count-- > 0) {
-		res = sin(phase) * 32767;
+		res = sin(phase) * maxval;
 		ires = res;
+		tmp = (unsigned char *)(&ires);
 		for (chn = 0; chn < channels; chn++) {
-			*samples[chn] = ires;
+			for (byte = 0; byte < (unsigned int)bps; byte++)
+				*(samples[chn] + byte) = tmp[byte];
 			samples[chn] += steps[chn];
 		}
 		phase += step;
@@ -73,6 +77,7 @@ static int set_hwparams(snd_pcm_t *handle,
 			snd_pcm_access_t access)
 {
 	unsigned int rrate;
+	snd_pcm_uframes_t size;
 	int err, dir;
 
 	/* choose all parameters */
@@ -122,22 +127,24 @@ static int set_hwparams(snd_pcm_t *handle,
 		printf("Unable to set buffer time %i for playback: %s\n", buffer_time, snd_strerror(err));
 		return err;
 	}
-	err = snd_pcm_hw_params_get_buffer_size(params, &buffer_size);
+	err = snd_pcm_hw_params_get_buffer_size(params, &size);
 	if (err < 0) {
 		printf("Unable to get buffer size for playback: %s\n", snd_strerror(err));
 		return err;
 	}
+	buffer_size = size;
 	/* set the period time */
 	err = snd_pcm_hw_params_set_period_time_near(handle, params, &period_time, &dir);
 	if (err < 0) {
 		printf("Unable to set period time %i for playback: %s\n", period_time, snd_strerror(err));
 		return err;
 	}
-	err = snd_pcm_hw_params_get_period_size(params, &period_size, &dir);
+	err = snd_pcm_hw_params_get_period_size(params, &size, &dir);
 	if (err < 0) {
 		printf("Unable to get period size for playback: %s\n", snd_strerror(err));
 		return err;
 	}
+	period_size = size;
 	/* write the parameters to device */
 	err = snd_pcm_hw_params(handle, params);
 	if (err < 0) {
@@ -500,8 +507,8 @@ static void async_direct_callback(snd_async_handler_t *ahandler)
 }
 
 static int async_direct_loop(snd_pcm_t *handle,
-			     signed short *samples,
-			     snd_pcm_channel_area_t *areas)
+			     signed short *samples ATTRIBUTE_UNUSED,
+			     snd_pcm_channel_area_t *areas ATTRIBUTE_UNUSED)
 {
 	struct async_private_data data;
 	snd_async_handler_t *ahandler;
@@ -558,8 +565,8 @@ static int async_direct_loop(snd_pcm_t *handle,
  */
 
 static int direct_loop(snd_pcm_t *handle,
-		       signed short *samples,
-		       snd_pcm_channel_area_t *areas)
+		       signed short *samples ATTRIBUTE_UNUSED,
+		       snd_pcm_channel_area_t *areas ATTRIBUTE_UNUSED)
 {
 	double phase = 0;
 	const snd_pcm_channel_area_t *my_areas;
@@ -708,10 +715,11 @@ static void help(void)
 "-b,--buffer	ring buffer size in us\n"
 "-p,--period	period size in us\n"
 "-m,--method	transfer method\n"
+"-o,--format	sample format\n"
 "-v,--verbose   show the PCM setup parameters\n"
 "\n");
         printf("Recognized sample formats are:");
-        for (k = 0; k < SND_PCM_FORMAT_LAST; ++(unsigned long) k) {
+        for (k = 0; k < SND_PCM_FORMAT_LAST; ++k) {
                 const char *s = snd_pcm_format_name(k);
                 if (s)
                         printf(" %s", s);
@@ -735,6 +743,7 @@ int main(int argc, char *argv[])
 		{"buffer", 1, NULL, 'b'},
 		{"period", 1, NULL, 'p'},
 		{"method", 1, NULL, 'm'},
+		{"format", 1, NULL, 'o'},
 		{"verbose", 1, NULL, 'v'},
 		{"noresample", 1, NULL, 'n'},
 		{NULL, 0, NULL, 0},
@@ -754,7 +763,7 @@ int main(int argc, char *argv[])
 	morehelp = 0;
 	while (1) {
 		int c;
-		if ((c = getopt_long(argc, argv, "hD:r:c:f:b:p:m:vn", long_option, NULL)) < 0)
+		if ((c = getopt_long(argc, argv, "hD:r:c:f:b:p:m:o:vn", long_option, NULL)) < 0)
 			break;
 		switch (c) {
 		case 'h':
@@ -790,11 +799,21 @@ int main(int argc, char *argv[])
 			break;
 		case 'm':
 			for (method = 0; transfer_methods[method].name; method++)
-				if (!strcasecmp(transfer_methods[method].name, optarg))
+					if (!strcasecmp(transfer_methods[method].name, optarg))
 					break;
 			if (transfer_methods[method].name == NULL)
 				method = 0;
 			break;
+		case 'o':
+			for (format = 0; format < SND_PCM_FORMAT_LAST; format++) {
+				const char *format_name = snd_pcm_format_name(format);
+				if (format_name)
+					if (!strcasecmp(format_name, optarg))
+					break;
+			}
+			if (format == SND_PCM_FORMAT_LAST)
+				format = SND_PCM_FORMAT_S16;
+			break;
 		case 'v':
 			verbose = 1;
 			break;
@@ -850,8 +869,8 @@ int main(int argc, char *argv[])
 	}
 	for (chn = 0; chn < channels; chn++) {
 		areas[chn].addr = samples;
-		areas[chn].first = chn * 16;
-		areas[chn].step = channels * 16;
+		areas[chn].first = chn * snd_pcm_format_width(format);
+		areas[chn].step = channels * snd_pcm_format_width(format);
 	}
 
 	err = transfer_methods[method].transfer_loop(handle, samples, areas);
